@@ -58,6 +58,7 @@ static GOptionEntry entries[] = {
 static gboolean
 timed_exit_cb (void)
 {
+        g_debug ("Doing timed exit");
         gtk_main_quit ();
         return FALSE;
 }
@@ -70,15 +71,33 @@ on_session_over (GDBusProxy *proxy,
                  gpointer    user_data)
 {
         if (g_strcmp0 (signal_name, "SessionOver") == 0) {
+                g_debug ("Got a SessionOver signal - stopping");
                 gnome_settings_manager_stop (manager);
                 gtk_main_quit ();
         }
 }
 
 static void
-got_session_proxy (GObject *source_object,
+on_client_registered (GObject             *source_object,
+                      GAsyncResult        *res,
+                      gpointer             user_data)
+{
+        GVariant *variant;
+        GError *error = NULL;
+
+        variant = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
+        if (error != NULL) {
+                g_warning ("Unable to register client: %s", error->message);
+                g_error_free (error);
+        } else {
+                g_variant_unref (variant);
+        }
+}
+
+static void
+got_session_proxy (GObject      *source_object,
                    GAsyncResult *res,
-                   gpointer user_data)
+                   gpointer      user_data)
 {
         GDBusProxy *proxy;
         GError *error = NULL;
@@ -88,8 +107,19 @@ got_session_proxy (GObject *source_object,
                 g_debug ("Could not connect to the Session manager: %s", error->message);
                 g_error_free (error);
         } else {
+                const char *startup_id;
+
                 g_signal_connect (G_OBJECT (proxy), "g-signal",
                                   G_CALLBACK (on_session_over), NULL);
+                startup_id = g_getenv ("DESKTOP_AUTOSTART_ID");
+                g_dbus_proxy_call (proxy,
+                                   "RegisterClient",
+                                   g_variant_new ("(ss)", "gnome-settings-daemon", startup_id),
+                                   G_DBUS_CALL_FLAGS_NONE,
+                                   -1,
+                                   NULL,
+                                   (GAsyncReadyCallback) on_client_registered,
+                                   manager);
         }
 }
 
@@ -104,6 +134,7 @@ on_term_signal_pipe_closed (GIOChannel *source,
 
         term_signal_pipe_fds[0] = -1;
 
+        g_debug ("Received SIGTERM - shutting down");
         /* Got SIGTERM, time to clean up and get out
          */
         gtk_main_quit ();
@@ -172,6 +203,8 @@ name_lost_handler (GDBusConnection *connection,
                    gpointer user_data)
 {
         /* Name was already taken, or the bus went away */
+
+        g_warning ("Name taken or bus went away - shutting down");
         gtk_main_quit ();
 }
 
@@ -301,6 +334,8 @@ main (int argc, char *argv[])
 
         gtk_main ();
 
+        g_debug ("Shutting down");
+
 out:
         if (name_id > 0) {
                 g_bus_unown_name (name_id);
