@@ -36,14 +36,11 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <libnotify/notify.h>
 
 #include <X11/XKBlib.h>
 #include <X11/extensions/XKBstr.h>
 
-#ifdef HAVE_LIBNOTIFY
-#include <libnotify/notify.h>
-#endif /* HAVE_LIBNOTIFY */
-
 #include "gnome-settings-profile.h"
 #include "gsd-a11y-keyboard-manager.h"
 #include "gsd-a11y-preferences-dialog.h"
@@ -67,9 +64,7 @@ struct GsdA11yKeyboardManagerPrivate
 
         GSettings        *settings;
 
-#ifdef HAVE_LIBNOTIFY
         NotifyNotification *notification;
-#endif /* HAVE_LIBNOTIFY */
 };
 
 static void     gsd_a11y_keyboard_manager_class_init  (GsdA11yKeyboardManagerClass *klass);
@@ -446,7 +441,6 @@ maybe_show_status_icon (GsdA11yKeyboardManager *manager)
         gtk_status_icon_set_visible (manager->priv->status_icon, show);
 }
 
-#ifdef HAVE_LIBNOTIFY
 static void
 on_notification_closed (NotifyNotification     *notification,
                         GsdA11yKeyboardManager *manager)
@@ -507,13 +501,11 @@ on_sticky_keys_action (NotifyNotification     *notification,
         }
 }
 
-#endif /* HAVE_LIBNOTIFY */
-
 static gboolean
 ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
                                  gboolean                enabled)
 {
-#ifdef HAVE_LIBNOTIFY
+#if 1
         gboolean    res;
         const char *title;
         const char *message;
@@ -570,9 +562,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         }
 
         return res;
-#else
-        return FALSE;
-#endif /* HAVE_LIBNOTIFY */
+#endif /* 1 */
 }
 
 
@@ -647,7 +637,7 @@ static gboolean
 ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
                                    gboolean                enabled)
 {
-#ifdef HAVE_LIBNOTIFY
+#if 1
         gboolean    res;
         const char *title;
         const char *message;
@@ -707,9 +697,7 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         }
 
         return res;
-#else
-        return FALSE;
-#endif /* HAVE_LIBNOTIFY */
+#endif /* 1 */
 }
 
 static void
@@ -996,6 +984,9 @@ gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
 static void
 restore_server_xkb_config (GsdA11yKeyboardManager *manager)
 {
+        if (manager->priv->original_xkb_desc == NULL) {
+                return;
+        }
         gdk_error_trap_push ();
         XkbSetControls (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                         XkbSlowKeysMask         |
@@ -1030,11 +1021,15 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
                 p->device_manager = NULL;
         }
 
-        if (p->status_icon)
+        if (p->status_icon) {
                 gtk_status_icon_set_visible (p->status_icon, FALSE);
+                p->status_icon = NULL;
+        }
 
-        g_object_unref (p->settings);
-        p->settings = NULL;
+        if (p->settings != NULL) {
+                g_object_unref (p->settings);
+                p->settings = NULL;
+        }
 
         gdk_window_remove_filter (NULL,
                                   (GdkFilterFunc) cb_xkb_event_filter,
@@ -1044,11 +1039,15 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
          */
         restore_server_xkb_config (manager);
 
-        if (p->slowkeys_alert != NULL)
+        if (p->slowkeys_alert != NULL) {
                 gtk_widget_destroy (p->slowkeys_alert);
+                p->slowkeys_alert = NULL;
+        }
 
-        if (p->stickykeys_alert != NULL)
+        if (p->stickykeys_alert != NULL) {
                 gtk_widget_destroy (p->stickykeys_alert);
+                p->stickykeys_alert = NULL;
+        }
 
         p->slowkeys_shortcut_val = FALSE;
         p->stickykeys_shortcut_val = FALSE;
@@ -1184,10 +1183,6 @@ static void
 gsd_a11y_keyboard_manager_init (GsdA11yKeyboardManager *manager)
 {
         manager->priv = GSD_A11Y_KEYBOARD_MANAGER_GET_PRIVATE (manager);
-
-#ifdef HAVE_LIBNOTIFY
-        notify_init ("gnome-settings-daemon");
-#endif /* HAVE_LIBNOTIFY */
 }
 
 static void
